@@ -1,22 +1,21 @@
 """Common functions for ecl2df modules"""
 
-import sys
-import json
-import re
-import signal
-import inspect
-import logging
-import datetime
 import argparse
+import datetime
+import inspect
 import itertools
-from pathlib import Path
+import json
+import logging
+import re
 import shlex
-from typing import Dict, Optional, List, Union, Set, Any
+import signal
+import sys
+from pathlib import Path
+from typing import Any, Dict, List, Optional, Set, Union
 
 import numpy as np
 import pandas as pd
 
-
 try:
     # pylint: disable=unused-import
     import opm.io.deck  # lgtm [py/import-and-import-from]
@@ -34,7 +33,9 @@ from ecl2df import __version__
 # of supported Eclipse keyword data
 OPMKEYWORDS: Dict[str, dict] = {}
 for keyw in [
+    "BRANPROP",
     "COMPDAT",
+    "COMPLUMP",
     "COMPSEGS",
     "DENSITY",
     "EQLDIMS",
@@ -42,6 +43,7 @@ for keyw in [
     "FAULTS",
     "GRUPNET",
     "GRUPTREE",
+    "NODEPROP",
     "PBVD",
     "PDVD",
     "PVDG",
@@ -68,6 +70,7 @@ for keyw in [
     "WELOPEN",
     "WELSEGS",
     "WELSPECS",
+    "WLIST",
     "WSEGAICD",
     "WSEGSICD",
     "WSEGVALV",
@@ -76,7 +79,6 @@ for keyw in [
         (Path(__file__).parent / "opmkeywords" / keyw).read_text()
     )
 
-
 # This is a magic filename that means read/write from/to stdout
 # This makes it impossible to write to a file called "-" on disk
 # but that would anyway create a lot of other problems in the shell.
@@ -116,6 +118,8 @@ def write_dframe_stdout_file(
         signal.signal(signal.SIGPIPE, signal.SIG_DFL)
         dframe.to_csv(sys.stdout, index=index)
     else:
+        if caller_logger and dframe.empty:
+            caller_logger.warning("Empty dataframe being written to disk")
         if caller_logger and not logstr:
             caller_logger.info("Writing to file %s", str(output))
         elif caller_logger and logstr:
@@ -239,7 +243,7 @@ def parse_opmio_deckrecord(
     itemlistname: str = "items",
     recordindex: Optional[int] = None,
     renamer: Optional[Union[Dict[str, str], Dict[str, Union[str, List[str]]]]] = None,
-) -> dict:
+) -> Dict[str, Any]:
     """
     Parse an opm.io.DeckRecord belonging to a certain keyword
 
@@ -259,7 +263,7 @@ def parse_opmio_deckrecord(
     if keyword not in OPMKEYWORDS:
         raise ValueError(f"Keyword {keyword} not supported by common.py")
 
-    rec_dict = {}
+    rec_dict: Dict[str, Any] = {}
 
     if recordindex is None:  # Beware, 0 is different from None here.
         itemlist = OPMKEYWORDS[keyword][itemlistname]
@@ -271,24 +275,44 @@ def parse_opmio_deckrecord(
     # but for some keywords there are more values, like for PVTO
     for item_idx, jsonitem in enumerate(itemlist):
         item_name = jsonitem["name"]
-        if not record[item_idx].defaulted:
+        try:
+            defaulted = record[item_idx].defaulted
+        except IndexError:
+            # Workaround for missing default in json for WLIST item WELLS (empty string)
+            defaulted = True
+        if not defaulted:
             if len(record[item_idx]) == 1:
                 # The DeckItem attribute .value is only present if there is an
                 # explicit statement "from opm.io.deck import DeckKeyword"
                 # in this file.
                 rec_dict[item_name] = record[item_idx].value
             else:
-                rec_dict[item_name] = record[item_idx].get_raw_data_list()
+                try:
+                    rec_dict[item_name] = record[item_idx].get_raw_data_list()
+                except ValueError:
+                    # Will get here for string lists:
+                    rec_dict[item_name] = record[item_idx].get_data_list()
                 # (the caller is responsible for unrolling this list with
                 # correct naming of elements)
+
+                # When we parse a list, some values in it can be defaulted, for
+                # which the default values are not provided in the JSON. Return
+                # these defaulted values as NaN's for the calling code to fix.
+                for idx in range(len(record[item_idx])):
+                    # This code is using a private attribute of an
+                    # OPM DeckItem. A better solution has not yet
+                    # been found in the OPM API. See also
+                    # https://github.com/OPM/opm-common/issues/2598
+                    if record[item_idx].__defaulted(idx):
+                        rec_dict[item_name][idx] = np.nan
         else:
             rec_dict[item_name] = jsonitem.get("default", None)
 
     if renamer:
-        renamed_dict = {}
+        renamed_dict: Dict[str, Any] = {}
         for key, value in rec_dict.items():
             if key in renamer and not isinstance(renamer[key], list):
-                renamed_dict[renamer[key]] = value
+                renamed_dict[renamer[key]] = value  # type: ignore
             else:
                 renamed_dict[key] = value
         return renamed_dict
@@ -639,7 +663,7 @@ def stack_on_colnames(
             of the column multiindex
     """
     if not inplace:
-        dframe = pd.DataFrame(dframe)
+        dframe = dframe.copy()
     tuplecolumns = list(map(lambda x: tuple(x.split(sep)), dframe.columns))
     if max(map(len, tuplecolumns)) < 2:
         logger.info("No columns to stack")
@@ -763,3 +787,34 @@ def convert_lyrlist_to_zonemap(lyrlist: List[Dict[str, Any]]) -> Dict[int, str]:
             to_layer = zonedict["to_layer"]
         zonemap.update(dict.fromkeys(range(from_layer, to_layer + 1), zonedict["name"]))
     return zonemap
+
+
+def get_wells_matching_template(template: str, wells: list):
+    """Returns the wells in the list that is matching the template
+    containing wilcard characters. The wildcard charachters supported
+    are * to match zero or more charachters and ? to match a single
+    non-empty character. Any wildcards in the beginning of the template
+    must be preceded with a \\.
+
+    Well name templates starting with ? might be allowed in Eclipse
+    in some contexts, but not in all and will not be permitted here to
+    avoid confusion.
+
+    Args:
+        template: well name template with wildcard characters
+        wells: list of wells to match against the template
+
+    Returns:
+        List of matched wells
+    """
+    if template.startswith("*") or template.startswith("?"):
+        raise ValueError(
+            "Well template not allowed to start with a wildcard character: "
+            f"Must be preceded with a \\: {template}"
+        )
+    elif template.startswith("\\"):
+        # Note that the two \\ are actually read as one and
+        # this will return True for f.ex '\*P1'
+        template = template[1:]
+    regex = template.replace("*", ".*").replace("?", ".")
+    return [well for well in wells if bool(re.match(regex, well))]
