@@ -4,8 +4,8 @@ Extract the contents of the FAULTS keyword into
 a DataFrame
 
 """
-import logging
 import argparse
+import logging
 from typing import Union
 
 import pandas as pd
@@ -87,9 +87,6 @@ def faults_main(args) -> None:
     if eclfiles:
         deck = eclfiles.get_ecldeck()
     faults_df = df(deck)
-    if faults_df.empty:
-        logger.warning("Empty FAULT data, not written to disk!")
-        return
     write_dframe_stdout_file(
         faults_df,
         args.output,
