@@ -9,21 +9,21 @@ geometric information. Static data (properties) can be merged from
 the INIT file, and dynamic data can be merged from the Restart (UNRST)
 file.
 """
-import logging
-import fnmatch
-import textwrap
 import argparse
 import datetime
+import fnmatch
+import logging
+import textwrap
 from pathlib import Path
-from typing import Union, List, Tuple, Optional, Dict, Type
+from typing import Dict, List, Optional, Tuple, Type, Union
 
 import dateutil.parser
-
 import numpy as np
 import pandas as pd
-
 from ecl.eclfile import EclFile
-from ecl2df import common, __version__
+
+from ecl2df import __version__, common
+
 from .eclfiles import EclFiles
 
 logger = logging.getLogger(__name__)
@@ -91,10 +91,11 @@ def dates2rstindices(
                 chosendates = [availabledates[-1]]
             elif dates == "all":
                 chosendates = availabledates
-    elif isinstance(dates, datetime.date):
-        chosendates = [dates]
     elif isinstance(dates, datetime.datetime):
         chosendates = [dates.date()]
+    elif isinstance(dates, datetime.date):
+        # Beware, datetime.datetime would also match here.
+        chosendates = [dates]
     elif isinstance(dates, list):
         chosendates = [x for x in dates if x in availabledates]
         if not chosendates:
@@ -273,8 +274,6 @@ def gridgeometry2df(
         pr. cell. The index of the dataframe are the global indices. If a zonemap
         is provided, zone information will be in the column ZONE.
     """
-    if not eclfiles:
-        raise ValueError
     egrid_file = eclfiles.get_egridfile()
     grid = eclfiles.get_egrid()
 
@@ -363,8 +362,11 @@ def merge_initvectors(
     assert isinstance(dframe, pd.DataFrame)
     assert isinstance(eclfiles, EclFiles)
 
-    for col in ijknames:
-        assert col in dframe
+    if not set(ijknames).issubset(dframe.columns):
+        raise ValueError(
+            f"All of the columns {ijknames} must be present "
+            "in the dataframe for merging"
+        )
 
     if isinstance(initvectors, str):
         initvectors = [initvectors]
