@@ -1,16 +1,16 @@
 """
 Extract non-neighbour connection (NNC) information from Eclipse output files.
 """
-import os
-import logging
-import datetime
 import argparse
+import datetime
+import logging
+import os
 from pathlib import Path
 from typing import Optional
 
 import pandas as pd
 
-from ecl2df import common, EclFiles, grid, __version__
+from ecl2df import EclFiles, __version__, common, grid
 from ecl2df.common import write_dframe_stdout_file
 
 logger: logging.Logger = logging.getLogger(__name__)
@@ -282,9 +282,6 @@ def nnc_main(args) -> None:
         logging.basicConfig(level=logging.INFO)
     eclfiles = EclFiles(args.DATAFILE)
     nncdf = df(eclfiles, coords=args.coords, pillars=args.pillars)
-    if nncdf.empty:
-        logger.warning("Empty NNC dataframe being written to disk!")
-        return
     write_dframe_stdout_file(
         nncdf,
         args.output,
