@@ -4,14 +4,14 @@ Extract the PVT data from an Eclipse (input) deck as Pandas Dataframes
 Data can be extracted from a full Eclipse deck or from individual files.
 """
 
-import logging
 import argparse
+import logging
 from pathlib import Path
-from typing import List, Dict, Union, Optional
+from typing import Dict, List, Optional, Union
 
 import pandas as pd
 
-from ecl2df import inferdims, common, EclFiles
+from ecl2df import EclFiles, common, inferdims
 
 try:
     # Needed for mypy
@@ -296,20 +296,19 @@ def pvt_main(args) -> None:
         # more error-prone, and it needs a string as input.
         stringdeck = Path(args.DATAFILE).read_text()
         pvt_df = df(stringdeck, keywords=args.keywords)
-    if not pvt_df.empty:
-        common.write_dframe_stdout_file(
-            pvt_df,
-            args.output,
-            index=False,
-            caller_logger=logger,
-            logstr=(
-                "Unique PVTNUMs: {}, PVT keywords: {}".format(
-                    str(len(pvt_df["PVTNUM"].unique())), str(pvt_df["KEYWORD"].unique())
-                )
-            ),
-        )
+    if "PVTNUM" in pvt_df and "KEYWORD" in pvt_df:
+        pvtnums = str(len(pvt_df["PVTNUM"].unique()))
+        keywords = str(pvt_df["KEYWORD"].unique())
     else:
-        logger.error("Empty PVT data, not written to disk")
+        pvtnums = "-"
+        keywords = "-"
+    common.write_dframe_stdout_file(
+        pvt_df,
+        args.output,
+        index=False,
+        caller_logger=logger,
+        logstr=f"Unique PVTNUMs: {pvtnums}, PVT keywords: {keywords}",
+    )
 
 
 def pvt_reverse_main(args) -> None:
