@@ -12,10 +12,10 @@ SATNUMs, but whenever this is known, it is recommended to either supply
 TABDIMS or to supply the satnumcount directly to avoid possible bugs.
 
 """
-import logging
 import argparse
+import logging
 from pathlib import Path
-from typing import List, Dict, Union, Optional
+from typing import Dict, List, Optional, Union
 
 import pandas as pd
 
@@ -25,9 +25,10 @@ try:
 except ImportError:
     pass
 
-from ecl2df import inferdims, common
-from .eclfiles import EclFiles
+from ecl2df import common, inferdims
+
 from .common import write_dframe_stdout_file
+from .eclfiles import EclFiles
 
 logger: logging.Logger = logging.getLogger(__name__)
 
@@ -56,26 +57,6 @@ RENAMERS["SWFN"] = {"DATA": ["SW", "KRW", "PCOW"]}
 RENAMERS["SWOF"] = {"DATA": ["SW", "KRW", "KROW", "PCOW"]}
 
 
-def xx_inject_satnumcount(deckstr: str, satnumcount: int) -> str:
-    """Insert a TABDIMS with NTSFUN into a deck
-
-    This is simple string manipulation, not OPM
-    deck manipulation (which might be possible to do).
-
-    Arguments:
-        deckstr: A string containing a partial deck (f.ex only
-            the SWOF keyword).
-        satnumcount: The NTSFUN number to use in TABDIMS
-            (this function does not care if it is correct or not)
-    Returns:
-        New deck with TABDIMS prepended.
-    """
-    if "TABDIMS" in deckstr:
-        logger.warning("Not inserting TABDIMS in a deck where already exists")
-        return deckstr
-    return "TABDIMS\n " + str(satnumcount) + " /\n\n" + str(deckstr)
-
-
 def df(
     deck: Union[str, "opm.libopmcommon_python.Deck"],
     keywords: Optional[List[str]] = None,
@@ -116,17 +97,18 @@ def df(
         deck = deck.get_ecldeck()
     deck = inferdims.inject_xxxdims_ntxxx("TABDIMS", "NTSFUN", deck, ntsfun)
     assert "TABDIMS" in deck
-    ntsfun = deck["TABDIMS"][0][inferdims.DIMS_POS["NTSFUN"]].get_int(0)
 
     wanted_keywords = common.handle_wanted_keywords(keywords, deck, SUPPORTED_KEYWORDS)
 
     frames = []
     for keyword in wanted_keywords:
-        # Construct the associated function names
-        function_name = keyword.lower() + "_fromdeck"
-        function = globals()[function_name]
-        dframe = function(deck, ntsfun=ntsfun)
-        frames.append(dframe.assign(KEYWORD=keyword))
+        frames.append(
+            interpolate_defaults(
+                common.ecl_keyworddata_to_df(
+                    deck, keyword, renamer=RENAMERS[keyword], recordcountername="SATNUM"
+                ).assign(KEYWORD=keyword)
+            )
+        )
     nonempty_frames = [frame for frame in frames if not frame.empty]
     if nonempty_frames:
         dframe = pd.concat(nonempty_frames, axis=0, sort=False, ignore_index=True)
@@ -140,140 +122,30 @@ def df(
     return pd.DataFrame()
 
 
-def swof_fromdeck(
-    deck: Union[str, "opm.libopmcommon_python.Deck"], ntsfun: Optional[int] = None
-) -> pd.DataFrame:
-    """Extract SWOF data from a deck
-
-    Args:
-        deck
-        ntsfun: Number of SATNUM regions in deck. Will
-            be inferred if not present in deck
+def interpolate_defaults(dframe: pd.DataFrame) -> pd.DataFrame:
+    """Interpolate NaN's linearly in saturation.
+    Saturation function tables in Eclipse decks can have certain values defaulted.
+    When parsed by common.ecl2df, these values are returned as np.nan.
+    The incoming dataframe must be associated to one keyword only, but
+    can consist of multiple SATNUMs.
     """
-    if "TABDIMS" not in deck:
-        deck = inferdims.inject_xxxdims_ntxxx("TABDIMS", "NTSFUN", deck, ntsfun)
-    return common.ecl_keyworddata_to_df(
-        deck, "SWOF", renamer=RENAMERS["SWOF"], recordcountername="SATNUM"
-    )
+    sat_cols: set = {"SW", "SO", "SG", "SL"}.intersection(dframe.columns)
+    assert (
+        len(sat_cols) == 1
+    ), f"Could not determine a single saturation column in {dframe.columns}"
+    sat_col = list(sat_cols)[0]
 
+    if dframe[sat_col].isna().any():
+        raise ValueError("nan in saturation column is not allowed")
 
-def sgof_fromdeck(
-    deck: Union[str, "opm.libopmcommon_python.Deck"], ntsfun: Optional[int] = None
-) -> pd.DataFrame:
-    """Extract SGOF data from a deck
-
-    Args:
-        deck
-        ntsfun: Number of SATNUM regions in deck. Will
-            be inferred if not present in deck
-    """
-    if "TABDIMS" not in deck:
-        deck = inferdims.inject_xxxdims_ntxxx("TABDIMS", "NTSFUN", deck, ntsfun)
-    return common.ecl_keyworddata_to_df(
-        deck, "SGOF", renamer=RENAMERS["SGOF"], recordcountername="SATNUM"
-    )
-
-
-def swfn_fromdeck(
-    deck: Union[str, "opm.libopmcommon_python.Deck"], ntsfun: Optional[int] = None
-) -> pd.DataFrame:
-    """Extract SWFN data from a deck
-
-    Args:
-        deck
-        ntsfun: Number of SATNUM regions in deck. Will
-            be inferred if not present in deck
-    """
-    if "TABDIMS" not in deck:
-        deck = inferdims.inject_xxxdims_ntxxx("TABDIMS", "NTSFUN", deck, ntsfun)
-    return common.ecl_keyworddata_to_df(
-        deck, "SWFN", renamer=RENAMERS["SWFN"], recordcountername="SATNUM"
-    )
-
-
-def sof2_fromdeck(
-    deck: Union[str, "opm.libopmcommon_python.Deck"], ntsfun: Optional[int] = None
-) -> pd.DataFrame:
-    """Extract SOF2 data from a deck
-
-    Args:
-        deck
-        ntsfun: Number of SATNUM regions in deck. Will
-            be inferred if not present in deck
-    """
-    if "TABDIMS" not in deck:
-        deck = inferdims.inject_xxxdims_ntxxx("TABDIMS", "NTSFUN", deck, ntsfun)
-    return common.ecl_keyworddata_to_df(
-        deck, "SOF2", renamer=RENAMERS["SOF2"], recordcountername="SATNUM"
-    )
-
-
-def sgfn_fromdeck(
-    deck: Union[str, "opm.libopmcommon_python.Deck"], ntsfun: Optional[int] = None
-) -> pd.DataFrame:
-    """Extract SGFN data from a deck
-
-    Args:
-        deck
-        ntsfun: Number of SATNUM regions in deck. Will
-            be inferred if not present in deck
-    """
-    if "TABDIMS" not in deck:
-        deck = inferdims.inject_xxxdims_ntxxx("TABDIMS", "NTSFUN", deck, ntsfun)
-    return common.ecl_keyworddata_to_df(
-        deck, "SGFN", renamer=RENAMERS["SGFN"], recordcountername="SATNUM"
-    )
-
-
-def sgwfn_fromdeck(
-    deck: Union[str, "opm.libopmcommon_python.Deck"], ntsfun: Optional[int] = None
-) -> pd.DataFrame:
-    """Extract SGWFN data from a deck
-
-    Args:
-        deck
-        ntsfun: Number of SATNUM regions in deck. Will
-            be inferred if not present in deck
-    """
-    if "TABDIMS" not in deck:
-        deck = inferdims.inject_xxxdims_ntxxx("TABDIMS", "NTSFUN", deck, ntsfun)
-    return common.ecl_keyworddata_to_df(
-        deck, "SGWFN", renamer=RENAMERS["SGWFN"], recordcountername="SATNUM"
-    )
-
-
-def sof3_fromdeck(
-    deck: Union[str, "opm.libopmcommon_python.Deck"], ntsfun: Optional[int] = None
-) -> pd.DataFrame:
-    """Extract SOF3 data from a deck
-
-    Args:
-        deck
-        ntsfun: Number of SATNUM regions in deck. Will
-            be inferred if not present in deck
-    """
-    if "TABDIMS" not in deck:
-        deck = inferdims.inject_xxxdims_ntxxx("TABDIMS", "NTSFUN", deck, ntsfun)
-    return common.ecl_keyworddata_to_df(
-        deck, "SOF3", renamer=RENAMERS["SOF3"], recordcountername="SATNUM"
-    )
-
-
-def slgof_fromdeck(
-    deck: Union[str, "opm.libopmcommon_python.Deck"], ntsfun: Optional[int] = None
-) -> pd.DataFrame:
-    """Extract SLGOF data from a deck
-
-    Args:
-        deck
-        ntsfun: Number of SATNUM regions in deck. Will
-            be inferred if not present in deck
-    """
-    if "TABDIMS" not in deck:
-        deck = inferdims.inject_xxxdims_ntxxx("TABDIMS", "NTSFUN", deck, ntsfun)
-    return common.ecl_keyworddata_to_df(
-        deck, "SLGOF", renamer=RENAMERS["SLGOF"], recordcountername="SATNUM"
-    )
+    filled_frames = []
+    for _, subframe in dframe.groupby("SATNUM"):
+        filled_frames.append(
+            subframe.set_index(sat_col)
+            .interpolate(method="index", limit_area="inside")
+            .reset_index()
+        )
+    return pd.concat(filled_frames)
 
 
 def fill_parser(parser: argparse.ArgumentParser) -> argparse.ArgumentParser:
@@ -325,19 +197,19 @@ def satfunc_main(args) -> None:
         # This might be an include file for which we have to infer/guess
         # TABDIMS. Then we send it to df() as a string
         satfunc_df = df(Path(args.DATAFILE).read_text(), keywords=args.keywords)
-    if not satfunc_df.empty:
-        write_dframe_stdout_file(
-            satfunc_df,
-            args.output,
-            index=False,
-            caller_logger=logger,
-            logstr="Unique SATNUMs: {}, saturation keywords: {}".format(
-                str(len(satfunc_df["SATNUM"].unique())),
-                str(satfunc_df["KEYWORD"].unique()),
-            ),
-        )
+    if "SATNUM" in satfunc_df and "KEYWORD" in satfunc_df:
+        satnums = str(len(satfunc_df["SATNUM"].unique()))
+        keywords = str(satfunc_df["KEYWORD"].unique())
     else:
-        logger.error("Empty saturation functions data, not written to disk!")
+        satnums = "-"
+        keywords = "-"
+    write_dframe_stdout_file(
+        satfunc_df,
+        args.output,
+        index=False,
+        caller_logger=logger,
+        logstr=f"Unique SATNUMs: {satnums}, saturation keywords: {keywords}",
+    )
 
 
 def satfunc_reverse_main(args) -> None:
