@@ -2,16 +2,16 @@
 """
 Extract transmissibility information from Eclipse output files as Dataframes.
 """
-import logging
 import argparse
-from typing import Union, List
+import logging
+from typing import List, Union
 
 import pandas as pd
 
 import ecl2df.grid
 import ecl2df.nnc
-
 from ecl2df.common import write_dframe_stdout_file
+
 from .eclfiles import EclFiles
 
 try:
@@ -97,7 +97,7 @@ def df(
 
     if onlykdir and onlyijdir:
         logger.warning(
-            "Filtering to both k and to ij simultaneously " "results in empty dataframe"
+            "Filtering to both k and to ij simultaneously results in empty dataframe"
         )
 
     grid_df = ecl2df.grid.df(eclfiles)
