@@ -1,23 +1,21 @@
 """Test module for ecl2df.grid"""
 
-import sys
 import datetime
+import sys
 from pathlib import Path
 
 import numpy as np
 import pandas as pd
-
 import pytest
 
-from ecl2df import grid, common
-from ecl2df import ecl2csv
+from ecl2df import common, ecl2csv, grid
 from ecl2df.eclfiles import EclFiles
 
 TESTDIR = Path(__file__).absolute().parent
 DATAFILE = str(TESTDIR / "data/reek/eclipse/model/2_R001_REEK-0.DATA")
 
 
-def test_gridgeometry2df():
+def test_gridgeometry2df(mocker):
     """Test that dataframes are produced"""
     eclfiles = EclFiles(DATAFILE)
     grid_geom = grid.gridgeometry2df(eclfiles)
@@ -42,6 +40,17 @@ def test_gridgeometry2df():
 
     assert (grid_geom["Z_MAX"] > grid_geom["Z_MIN"]).all()
 
+    with pytest.raises(TypeError, match="missing 1 required positional"):
+        grid.gridgeometry2df()
+
+    with pytest.raises(AttributeError):
+        # This error situation we don't really try to handle.
+        grid.gridgeometry2df(None)
+
+    with pytest.raises(ValueError, match="No EGRID file supplied"):
+        mocker.patch("ecl2df.eclfiles.EclFiles.get_egridfile", return_value=None)
+        grid.gridgeometry2df(eclfiles)
+
 
 def test_wrongfile():
     """Test the EclFiles object on nonexistent files"""
@@ -94,6 +103,36 @@ def test_gridzonemap():
     assert len(dframe) == len(grid_no_zone)
 
 
+def test_merge_initvectors():
+    eclfiles = EclFiles(DATAFILE)
+    assert grid.merge_initvectors(eclfiles, pd.DataFrame(), []).empty
+    foo_df = pd.DataFrame([{"FOO": 1}])
+    pd.testing.assert_frame_equal(grid.merge_initvectors(eclfiles, foo_df, []), foo_df)
+
+    with pytest.raises(ValueError, match="All of the columns"):
+        grid.merge_initvectors(eclfiles, foo_df, ["NONEXISTING"])
+
+    minimal_df = pd.DataFrame([{"I": 10, "J": 11, "K": 12}])
+
+    with pytest.raises(KeyError):
+        grid.merge_initvectors(eclfiles, minimal_df, ["NONEXISTING"])
+
+    withporo = grid.merge_initvectors(eclfiles, minimal_df, ["PORO"])
+    pd.testing.assert_frame_equal(
+        withporo, minimal_df.assign(PORO=0.221848), check_dtype=False
+    )
+
+    with pytest.raises(ValueError):
+        # ijknames must be length 3
+        grid.merge_initvectors(
+            eclfiles, minimal_df, ["PORO"], ijknames=["I", "J", "K", "L"]
+        )
+    with pytest.raises(ValueError):
+        grid.merge_initvectors(eclfiles, minimal_df, ["PORO"], ijknames=["I", "J"])
+    with pytest.raises(ValueError, match="All of the columns"):
+        grid.merge_initvectors(eclfiles, minimal_df, ["PORO"], ijknames=["A", "B", "C"])
+
+
 def test_init2df():
     """Test that dataframe with INIT vectors can be produced"""
     eclfiles = EclFiles(DATAFILE)
@@ -145,6 +184,12 @@ def test_df2ecl(tmpdir):
     grid_df = grid.df(eclfiles)
 
     fipnum_str = grid.df2ecl(grid_df, "FIPNUM", dtype=int)
+    assert grid.df2ecl(grid_df, "FIPNUM", dtype="int", nocomments=True) == grid.df2ecl(
+        grid_df, "FIPNUM", dtype=int, nocomments=True
+    )
+    with pytest.raises(ValueError, match="Wrong dtype argument foo"):
+        grid.df2ecl(grid_df, "FIPNUM", dtype="foo")
+
     assert "FIPNUM" in fipnum_str
     assert "-- Output file printed by ecl2df.grid" in fipnum_str
     assert "35817 active cells" in fipnum_str  # (comment at the end)
@@ -177,6 +222,13 @@ def test_df2ecl(tmpdir):
     incstring = open("perm.inc").readlines()
     assert sum([1 for line in incstring if "PERM" in line]) == 6
 
+    assert grid.df2ecl(grid_df, ["PERMX"], dtype=float, nocomments=True) == grid.df2ecl(
+        grid_df, ["PERMX"], dtype="float", nocomments=True
+    )
+
+    # with pytest.raises(ValueError, match="Wrong dtype argument"):
+    grid.df2ecl(grid_df, ["PERMX"], dtype=dict)
+
     with pytest.raises(ValueError):
         grid.df2ecl(grid_df, ["PERMRR"])
 
@@ -191,6 +243,12 @@ def test_df2ecl(tmpdir):
     fipnum_str_rst = grid.df2ecl(gr_rst_stacked, "FIPNUM", dtype=int, nocomments=True)
     assert fipnum_str_rst == fipnum_str_nocomment
 
+    # dateinheaders here will be ignored due to stackdates:
+    pd.testing.assert_frame_equal(
+        gr_rst_stacked,
+        grid.df(eclfiles, rstdates="all", stackdates=True, dateinheaders=True),
+    )
+
 
 def test_df2ecl_mock():
     """Test that we can use df2ecl for mocked minimal dataframes"""
@@ -239,6 +297,15 @@ def test_dropconstants():
     assert "B" in grid.drop_constant_columns(df, alwayskeep="B")
     assert "B" in grid.drop_constant_columns(df, alwayskeep=["B"])
 
+    with pytest.raises(TypeError):
+        grid.drop_constant_columns(df, alwayskeep={"FOO"})
+    with pytest.raises(TypeError):
+        grid.drop_constant_columns(df, alwayskeep=1)
+    with pytest.raises(TypeError):
+        grid.drop_constant_columns({})
+
+    assert grid.drop_constant_columns(pd.DataFrame()).empty
+
 
 def test_df():
     """Test the df function"""
@@ -368,6 +435,7 @@ def test_main(tmpdir):
     sys.argv = [
         "ecl2csv",
         "grid",
+        "--verbose",
         DATAFILE,
         "-o",
         str(tmpcsvfile),
@@ -412,6 +480,47 @@ def test_get_available_rst_dates():
     assert didx[1][0] == alldates[0]
     assert didx[1][-1] == alldates[-1]
 
+    somedate = grid.dates2rstindices(eclfiles, "2000-07-01")
+    assert somedate[1] == [alldates[1]]
+
+    with pytest.raises(ValueError, match="date 1999-09-09 not found in UNRST file"):
+        grid.dates2rstindices(eclfiles, "1999-09-09")
+
+    with pytest.raises(ValueError, match="date 1999-0909 not understood"):
+        grid.dates2rstindices(eclfiles, "1999-0909")
+
+    expl_date = grid.dates2rstindices(eclfiles, datetime.date(2000, 7, 1))
+    assert expl_date[1] == [alldates[1]]
+
+    expl_datetime = grid.dates2rstindices(
+        eclfiles, datetime.datetime(2000, 7, 1, 0, 0, 0)
+    )
+    assert expl_datetime[1] == [alldates[1]]
+
+    expl_list_datetime = grid.dates2rstindices(eclfiles, [datetime.date(2000, 7, 1)])
+    assert expl_list_datetime[1] == [alldates[1]]
+
+    # For list input, only datetime.date objects are allowed:
+    expl_list2_date = grid.dates2rstindices(
+        eclfiles, [datetime.date(2000, 7, 1), datetime.date(2001, 2, 1)]
+    )
+    assert expl_list2_date[1] == [alldates[1], alldates[2]]
+
+    with pytest.raises(ValueError, match="None of the requested dates were found"):
+        grid.dates2rstindices(eclfiles, ["2000-07-01", "2001-02-01"])
+
+    with pytest.raises(ValueError, match="None of the requested dates were found"):
+        grid.dates2rstindices(
+            eclfiles,
+            [
+                datetime.datetime(2000, 7, 1, 0, 0, 0),
+                datetime.datetime(2001, 2, 1, 0, 0, 0),
+            ],
+        )
+
+    with pytest.raises(ValueError, match="not understood"):
+        grid.dates2rstindices(eclfiles, {"2000-07-01": "2001-02-01"})
+
     first = grid.dates2rstindices(eclfiles, "first")
     assert first[1][0] == alldates[0]
 
