@@ -2,10 +2,11 @@
 
 from pathlib import Path
 
-import pytest
+import numpy as np
 import pandas as pd
+import pytest
 
-from ecl2df import gruptree, ecl2csv
+from ecl2df import ecl2csv, gruptree
 from ecl2df.eclfiles import EclFiles
 
 TESTDIR = Path(__file__).absolute().parent
@@ -392,7 +393,7 @@ def test_main(tmpdir, mocker):
     assert not disk_df.empty
 
 
-def test_prettyprint(mocker, capsys):
+def test_prettyprint_commandline(mocker, capsys):
     """Test pretty printing via command line interface"""
     mocker.patch("sys.argv", ["ecl2csv", "gruptree", DATAFILE, "--prettyprint"])
     ecl2csv.main()
@@ -401,12 +402,14 @@ def test_prettyprint(mocker, capsys):
     assert (
         """
 Date: 2000-01-01
+GRUPTREE trees:
 FIELD
 ├── OP
 └── WI
 
 
 Date: 2000-02-01
+GRUPTREE trees:
 FIELD
 ├── OP
 │   ├── OP_1
@@ -417,6 +420,7 @@ FIELD
 
 
 Date: 2000-06-01
+GRUPTREE trees:
 FIELD
 ├── OP
 │   ├── OP_1
@@ -428,6 +432,7 @@ FIELD
 
 
 Date: 2001-01-01
+GRUPTREE trees:
 FIELD
 ├── OP
 │   ├── OP_1
@@ -441,6 +446,7 @@ FIELD
 
 
 Date: 2001-03-01
+GRUPTREE trees:
 FIELD
 ├── OP
 │   ├── OP_1
@@ -468,3 +474,236 @@ def test_main_subparser(tmpdir, mocker):
     assert Path(tmpcsvfile).is_file()
     disk_df = pd.read_csv(str(tmpcsvfile))
     assert not disk_df.empty
+
+
+@pytest.mark.parametrize(
+    "schstr, expected_dframe, check_columns",
+    [
+        (
+            # Changing BRANPROP
+            """
+DATES
+  1 JAN 2000 /
+/
+GRUPTREE
+ 'TMPL_A' 'FIELD'/
+/
+BRANPROP
+  'NODE_A'  'FIELD'  1 /
+  'TMPL_A'  'NODE_A' 2 /
+/
+NODEPROP
+  'FIELD'  20 /
+  'TMPL_A'  2*  YES /
+/
+DATES
+  1 FEB 2000 /
+/
+BRANPROP
+  'NODE_B'  'FIELD'  3 /
+  'TMPL_A'  'NODE_B' 4 /
+/
+        """,
+            pd.DataFrame(
+                [
+                    ["2000-01-01", "FIELD", "GRUPTREE", np.nan, np.nan, np.nan],
+                    ["2000-01-01", "TMPL_A", "GRUPTREE", "FIELD", np.nan, np.nan],
+                    ["2000-01-01", "FIELD", "BRANPROP", np.nan, np.nan, 20],
+                    ["2000-01-01", "NODE_A", "BRANPROP", "FIELD", 1, np.nan],
+                    ["2000-01-01", "TMPL_A", "BRANPROP", "NODE_A", 2, np.nan],
+                    ["2000-02-01", "FIELD", "BRANPROP", np.nan, np.nan, 20],
+                    ["2000-02-01", "NODE_A", "BRANPROP", "FIELD", 1, np.nan],
+                    ["2000-02-01", "NODE_B", "BRANPROP", "FIELD", 3, np.nan],
+                    ["2000-02-01", "TMPL_A", "BRANPROP", "NODE_B", 4, np.nan],
+                ],
+                columns=[
+                    "DATE",
+                    "CHILD",
+                    "KEYWORD",
+                    "PARENT",
+                    "VFP_TABLE",
+                    "TERMINAL_PRESSURE",
+                ],
+            ),
+            ["DATE", "CHILD", "KEYWORD", "PARENT", "VFP_TABLE", "TERMINAL_PRESSURE"],
+        ),
+        # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
+        (
+            # Changing NODEPROP
+            """
+DATES
+  1 JAN 2000 /
+/
+GRUPTREE
+ 'TMPL_A' 'FIELD'/
+/
+BRANPROP
+  'NODE_A'  'FIELD'  /
+  'TMPL_A'  'NODE_A'  /
+/
+NODEPROP
+  'FIELD'  20 /
+  'TMPL_A'  2*  YES /
+/
+DATES
+  1 FEB 2000 /
+/
+NODEPROP
+  'FIELD'  22  1* YES /
+/
+        """,
+            pd.DataFrame(
+                [
+                    ["2000-01-01", "FIELD", "GRUPTREE", np.nan, np.nan, np.nan],
+                    ["2000-01-01", "TMPL_A", "GRUPTREE", "FIELD", np.nan, np.nan],
+                    ["2000-01-01", "FIELD", "BRANPROP", np.nan, 20, "NO"],
+                    ["2000-01-01", "NODE_A", "BRANPROP", "FIELD", np.nan, np.nan],
+                    ["2000-01-01", "TMPL_A", "BRANPROP", "NODE_A", np.nan, "YES"],
+                    ["2000-02-01", "FIELD", "BRANPROP", np.nan, 22, "YES"],
+                    ["2000-02-01", "NODE_A", "BRANPROP", "FIELD", np.nan, np.nan],
+                    ["2000-02-01", "TMPL_A", "BRANPROP", "NODE_A", np.nan, "YES"],
+                ],
+                columns=[
+                    "DATE",
+                    "CHILD",
+                    "KEYWORD",
+                    "PARENT",
+                    "TERMINAL_PRESSURE",
+                    "ADD_GAS_LIFT_GAS",
+                ],
+            ),
+            [
+                "DATE",
+                "CHILD",
+                "KEYWORD",
+                "PARENT",
+                "TERMINAL_PRESSURE",
+                "ADD_GAS_LIFT_GAS",
+            ],
+        ),
+        # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
+        (
+            # WELSPECS
+            """
+DATES
+  1 JAN 2000 /
+/
+GRUPTREE
+ 'TMPL_A' 'FIELD'/
+/
+BRANPROP
+  'NODE_A'  'FIELD'  /
+  'TMPL_A'  'NODE_A'  /
+/
+NODEPROP
+  'FIELD'  20 /
+  'TMPL_A'  2*  YES /
+/
+WELSPECS
+  'WELL_1'  'TMPL_A' 1 1 1 OIL /
+  'WELL_2'  'TMPL_B' 1 1 1 OIL /
+/
+DATES
+  1 FEB 2000 /
+/
+NODEPROP
+  'FIELD' 22 /
+/
+        """,
+            # TMPL_B is not in any trees. The WELSPECS line is added
+            # only when there is a new GRUPTREE tree
+            # TMPL_A is in both trees, but is not repeated at the date
+            # where there is two trees
+            pd.DataFrame(
+                [
+                    ["2000-01-01", "FIELD", "GRUPTREE", np.nan],
+                    ["2000-01-01", "TMPL_A", "GRUPTREE", "FIELD"],
+                    ["2000-01-01", "WELL_2", "WELSPECS", "TMPL_B"],
+                    ["2000-01-01", "FIELD", "BRANPROP", np.nan],
+                    ["2000-01-01", "NODE_A", "BRANPROP", "FIELD"],
+                    ["2000-01-01", "TMPL_A", "BRANPROP", "NODE_A"],
+                    ["2000-01-01", "WELL_1", "WELSPECS", "TMPL_A"],
+                    ["2000-02-01", "FIELD", "BRANPROP", np.nan],
+                    ["2000-02-01", "NODE_A", "BRANPROP", "FIELD"],
+                    ["2000-02-01", "TMPL_A", "BRANPROP", "NODE_A"],
+                    ["2000-02-01", "WELL_1", "WELSPECS", "TMPL_A"],
+                ],
+                columns=["DATE", "CHILD", "KEYWORD", "PARENT"],
+            ),
+            ["DATE", "CHILD", "KEYWORD", "PARENT"],
+        ),
+        # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
+    ],
+)
+def test_branprop_nodeprop(schstr, expected_dframe, check_columns):
+    """Testing that the gruptree dataframe works correctly
+    when the schedule string contains BRANPROP and NODEPROP
+    """
+    deck = EclFiles.str2deck(schstr)
+    dframe = gruptree.df(deck).reset_index()
+    expected_dframe.DATE = pd.to_datetime(expected_dframe.DATE)
+    pd.testing.assert_frame_equal(
+        dframe[check_columns],
+        expected_dframe[check_columns],
+        check_dtype=False,
+    )
+
+
+def test_prettyprint(tmpdir, mocker, caplog):
+    """ "Test prettyprinting with multiple dates and both
+    GRUPTREE and BRANPROP trees"""
+    schstr = """
+DATES
+  1 JAN 2000 /
+/
+GRUPTREE
+ 'TMPL_A' 'FIELD'/
+/
+BRANPROP
+  'NODE_A'  'FIELD'  /
+  'TMPL_A'  'NODE_A'  /
+/
+NODEPROP
+  'FIELD'  20 /
+  'TMPL_A'  2*  YES /
+/
+WELSPECS
+  'WELL_1'  'TMPL_A' 1 1 1 OIL /
+  'WELL_2'  'TMPL_B' 1 1 1 OIL /
+/
+DATES
+  1 FEB 2000 /
+/
+NODEPROP
+  'FIELD' 22 /
+/
+    """
+
+    expected_prettyprint = """
+Date: 2000-01-01
+GRUPTREE trees:
+FIELD
+└── TMPL_A
+    └── WELL_1
+
+TMPL_B
+└── WELL_2
+
+BRANPROP trees:
+FIELD
+└── NODE_A
+    └── TMPL_A
+        └── WELL_1
+
+
+Date: 2000-02-01
+BRANPROP trees:
+FIELD
+└── NODE_A
+    └── TMPL_A
+        └── WELL_1
+
+
+    """
+    dframe = gruptree.df(EclFiles.str2deck(schstr))
+    assert gruptree.prettyprint(dframe).strip() == expected_prettyprint.strip()
